@@ -1,4 +1,9 @@
-"""Utility module for optimization of hyperparameters
+"""
+==============================================================
+Hyperparameter optimization helpers (:mod:`reservoirpy.hyper`)
+==============================================================
+
+Utility module for optimization of hyperparameters
 using *hyperopt*.
 
 Note
@@ -9,6 +14,13 @@ and *Matplotlib* packages, which are not installed
 with ReservoirPy by default. Before using the
 :py:mod:`reservoirpy.hyper` module, consider installing
 these packages if they are not already installed.
+
+
+.. autosummary::
+    :toctree: generated/
+
+    research
+    plot_hyperopt_report
 """
 from ._hypersearch import research
 from ._hyperplot import plot_hyperopt_report
